@@ -66,22 +66,21 @@ Rcpp::List dlmtreeHDLMMGaussian(const Rcpp::List model){
   ctr->Zw = ctr->Z; 
   ctr->pZ = (ctr->Z).cols(); 
 
-  ctr->VgInv  = (ctr->Z).transpose() * (ctr->Z); 
+  ctr->Zt    = (ctr->Z).transpose();
+  ctr->VgInv  = ctr->Zt * (ctr->Z); 
   ctr->VgInv.diagonal().array() += 1.0 / 100000.0;
-  ctr->Vg     = ctr->VgInv.inverse();  
-  ctr->VgChol = (ctr->Vg).llt().matrixL();      
+  ctr->Vg     = ctr->VgInv.inverse();
+  ctr->VgChol = (ctr->Vg).llt().matrixL();
 
   // Binomial flag              
-  ctr->binomial = as<bool>(model["binomial"]);  
-  ctr->zinb     = as<bool>(model["zinb"]);  
+  ctr->binomial = 0;  
+  ctr->zinb     = 0;  
 
   // Tree prior probabilities for the modifier tree and DLM tree
   ctr->stepProbMod  = as<std::vector<double>>(model["stepProbMod"]);
-  ctr->stepProb     = as<std::vector<double>>(model["stepProbTDLM"]);
   ctr->treePriorMod = as<std::vector<double>>(model["treePriorMod"]); 
   ctr->treePrior    = as<std::vector<double>>(model["treePriorTDLM"]);
 
-
   // Diagnostics & messages
   ctr->verbose      = bool(model["verbose"]); 
   ctr->diagnostics  = bool(model["diagnostics"]);
@@ -94,13 +93,13 @@ Rcpp::List dlmtreeHDLMMGaussian(const Rcpp::List model){
   // 0 = none
   ctr->shrinkage = as<int>(model["shrinkage"]);
 
-  // Sparsity hyperparameter (Deprecated now as HDLMM does not perform exposure selection)
+  // Sparsity hyperparameter 
   ctr->mixKappa = as<double>(model["mixPrior"]); 
-  // bool updateKappa = false;  
-  // if (ctr->mixKappa < 0) {       
-  //   updateKappa = true;
-  //   ctr->mixKappa = 1;
-  // }
+  bool updateKappa = true;  
+  if (ctr->mixKappa < 0) {       
+    updateKappa = true;
+    ctr->mixKappa = 1;
+  }
 
   // *** Setup modifier data ***
   modDat *Mod = new modDat(as<std::vector<int>>(model["modIsNum"]), 
@@ -331,8 +330,8 @@ Rcpp::List dlmtreeHDLMMGaussian(const Rcpp::List model){
     ctr->sumTermT2  = (ctr->sumTermT2Exp).sum();
     ctr->totTerm    = (ctr->totTermExp).sum();
     if(ctr->interaction > 0) {
-      ctr->totTerm += (ctr->totTermMix).sum();
-      ctr->sumTermT2 += (ctr->sumTermT2Mix).sum();
+      ctr->totTerm    += (ctr->totTermMix).sum();
+      ctr->sumTermT2  += (ctr->sumTermT2Mix).sum();
     }
 
     // Update gamma
@@ -344,14 +343,16 @@ Rcpp::List dlmtreeHDLMMGaussian(const Rcpp::List model){
     // 1 = exposures/interactions (mu(s)) - (default)
     // 0 = none
     // Global shrinkage: nu
-    double xiInv  = R::rgamma(1, 1.0 / (1.0 + 1.0 / (ctr->nu)));
-    ctr->nu       = 1.0 / R::rgamma(0.5 * ctr->totTerm + 0.5,
-                              1.0 / (0.5 * ctr->sumTermT2 / (ctr->sigma2) + xiInv));
+
+    double xiInv  = R::rgamma(1, 1.0 / (1.0 + 1.0 / (ctr->nu)));   
+    ctr->nu = 1.0 / R::rgamma(0.5 * ctr->totTerm + 0.5,
+              1.0 / (0.5 * ctr->sumTermT2 / (ctr->sigma2) + xiInv));
                               
     // Exposure shrinkage
     double sigmanu = ctr->sigma2 * ctr->nu;
     if (ctr->shrinkage == 1 || ctr->shrinkage == 3) { 
       for (int i = 0; i < ctr->nExp; i++) {
+      
         xiInv         = R::rgamma(1, 1.0 / (1.0 + 1.0 / (ctr->muExp(i))));
         ctr->muExp(i) = 1.0 / R::rgamma(0.5 * ctr->totTermExp(i) + 0.5,
                         1.0 / (0.5 * ctr->sumTermT2Exp(i) / sigmanu + xiInv));
@@ -384,6 +385,18 @@ Rcpp::List dlmtreeHDLMMGaussian(const Rcpp::List model){
       Mod->modProb = rDirichlet((ctr->modCount.array() + ctr->modKappa / ctr->pM).matrix());  // (m_1 + kappa / J, ..., m_J + kappa / J)
       // end modifier selection
 
+      // HDLMM: Exposure selection posterior calculation
+      if (updateKappa) {
+        double mixKappaNew = R::rgamma(1.0, ctr->nTrees/4.0);
+        double mhrDirMix =
+          logDirichletDensity(ctr->expProb, ((ctr->expCount).array() + mixKappaNew).matrix()) -
+          logDirichletDensity(ctr->expProb, ((ctr->expCount).array() + ctr->mixKappa).matrix());
+
+        if (log(R::runif(0, 1)) < mhrDirMix && (mhrDirMix == mhrDirMix)){
+          ctr->mixKappa = mixKappaNew;
+        }
+      }
+
       // HDLMM: Exposure selection posterior calculation      
       ctr->expProb = rDirichlet(((ctr->expCount).array() + ctr->mixKappa).matrix());
 
@@ -577,7 +590,7 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
   treeMHR mhr0, mhr;                                  
 
   // Pre-calculation for MH ratio update
-  Eigen::VectorXd ZtR = (ctr->Z).transpose() * (ctr->R);
+  Eigen::VectorXd ZtR = ctr->Zt * (ctr->R);
 
   // -- List terminal nodes --
   modTerm   = modTree->listTerminal();   
@@ -605,238 +618,6 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
                           ctr, ZtR, treeVar, 
                           m1Var, m2Var, mixVar);
 
-  // // [Tree pair update]
-  // // *** Propose a new TDLMM tree 1 ***
-  // newExp    = m1; 
-  // newExpVar = m1Var;
-  // newMixVar = mixVar;
-  // stepMhr = 0;
-  // success = 0;
-
-  // // *** Propose a new dlmtree 1 ***
-  // // Choose a transition and update
-  // step1 = sampleInt(ctr->stepProb, 1);
-  // if ((dlmTerm1.size() == 1) && (step1 < 3)){
-  //   step1 = 0;
-  // }
-
-  // if(step1 < 3){ 
-  //   stepMhr = tdlmProposeTree(dlmTree1, Exp[m1], ctr, step1);     // Given current tree -> propose a new tree
-  //   success = dlmTree1->isProposed();                             // Update to proposed
-  //   newDlmTerm1 = dlmTree1->listTerminal(1);                      // If proposed, list the terminal nodes of the tree
-  // } else {
-  //   newExp = sampleInt(ctr->expProb);         // Sample an exposure
-  //   if (newExp != m1) {                       // If new Exposure is not the same as the previous one,
-  //     success   = 1;                          // It is good
-  //     newExpVar = ctr->muExp(newExp);         // Find the exposure-specific variance for the new exposure
-  //     newTree   = new Node(*dlmTree1);        // We need a new tree for a new exposure
-  //     newTree->setUpdate(1);                  // Set an update flag + for children as well (This is used for "update NodeVals")
-  //     newDlmTerm1 = newTree->listTerminal();  // List the terminal node of this new tree
-
-  //     for (Node* nt : newDlmTerm1)            // Go through the new terminal node
-  //       Exp[newExp]->updateNodeVals(nt);      // Update the math terms
-
-  //     // Update the interaction using the new exposure as well for TDLMMns/all
-  //     if ((ctr->interaction) && ((ctr->interaction == 2) || (newExp != m2))) {
-  //       if (newExp <= m2)
-  //         newMixVar = ctr->muMix(m2, newExp); 
-  //       else
-  //         newMixVar = ctr->muMix(newExp, m2); 
-  //     } else {
-  //       newMixVar = 0;
-  //     }
-  //   }
-  // }
-
-  // // * dlmTree 1 MHR 
-  // // StepMHR: Transition ratio
-  // // mhr0 & mhr: Likelihood of R
-  // // If successful, calculate MH ratio
-  // if (success) {  
-  //   // If transition is successful, we have a new dlm terminal node.
-  //   newDlmTerm1 = dlmTree1->listTerminal(1);
-  //   modTree->setUpdateXmat(1);
-
-  //   // MH ratio with a new terminal and the exposure: newDlmTerm1, newExpVar
-  //   mhr = dlmtreeHDLMM_MHR(modTerm, 
-  //                          newDlmTerm1, dlmTerm2, ctr, ZtR, treeVar, 
-  //                          newExpVar, m2Var, newMixVar);
-
-  //   // MH ratio - dlmTree 
-  //   if (RtR < 0) {
-  //     RtR = (ctr->R).dot(ctr->R);
-  //     RtZVgZtR = ZtR.dot((ctr->Vg).selfadjointView<Eigen::Lower>() * ZtR);
-  //   }
-
-  //   // MH ratio
-  //   ratio = stepMhr + 
-  //           mhr.logVThetaChol - mhr0.logVThetaChol -
-  //           (0.5 * (ctr->n + 1.0) *
-  //           (log(0.5 * (RtR - RtZVgZtR - mhr.beta) + ctr->xiInvSigma2) -
-  //           log(0.5 * (RtR - RtZVgZtR - mhr0.beta) + ctr->xiInvSigma2))) -
-  //           (0.5 * ((mhr.nTerm1 * mhr0.nModTerm * log(treeVar * newExpVar)) -
-  //           (mhr0.nTerm1 * mhr0.nModTerm * log(treeVar * m1Var)))); 
-
-  //   // Interaction
-  //   if (newMixVar != 0){ 
-  //     ratio -= 0.5 * log(treeVar * newMixVar) * mhr.nTerm1 * mhr0.nTerm2 * mhr0.nModTerm;
-  //   }
-
-  //   if (mixVar != 0){ 
-  //     ratio += 0.5 * log(treeVar * mixVar) * mhr0.nTerm1 * mhr0.nTerm2 * mhr0.nModTerm;
-  //   }
-
-  //   // Accept / Reject
-  //   if ((log(R::runif(0, 1)) < ratio) && (ratio == ratio)) {
-  //     mhr0 = mhr;
-  //     success = 2;
-
-  //     if (step1 == 3) { 
-  //       m1      = newExp;
-  //       m1Var   = newExpVar;
-  //       mixVar  = newMixVar;
-  //       dlmTree1->replaceNodeVals(newTree);
-  //     } else {
-  //       dlmTree1->accept(); 
-  //     }
-  //     // Update parameters with new tree
-  //     dlmTerm1 = dlmTree1->listTerminal();
-  //     for (Node* n : modTerm) {
-  //       n->nodevals->updateXmat = 0;
-  //     }
-
-  //   } else {  
-  //     modTree->setUpdateXmat(1); 
-  //   }
-  // }
-  // dlmTree1->reject();
-
-  // // Reset new tree value for tree2
-  // if (newTree != 0)
-  //   delete newTree;
-  // newTree = 0;
-
-  // // * Record tree 1
-  // if (ctr->diagnostics) {
-  //   Eigen::VectorXd acc(9);
-  //   acc << ctr->record, t, 1, step1, success, m1, dlmTerm1.size(), stepMhr, ratio;
-  //   (dgn->treeDLMAccept).push_back(acc);
-  // }
-
-  
-  // // *** Propose a new TDLMM tree 2 ***
-  // newExp    = m2; 
-  // newExpVar = m2Var;
-  // newMixVar = mixVar;
-  // stepMhr = 0;
-  // success = 0;
-
-  // // Choose a transition and update
-  // step2 = sampleInt(ctr->stepProb, 1);
-  // if ((dlmTerm2.size() == 1) && (step2 < 3)){
-  //   step2 = 0;
-  // }
-
-  // if(step2 < 3){ 
-  //   stepMhr = tdlmProposeTree(dlmTree2, Exp[m2], ctr, step2);
-  //   success = dlmTree2->isProposed();
-  //   newDlmTerm2 = dlmTree2->listTerminal(1);
-
-  // } else { 
-  //   newExp = sampleInt(ctr->expProb); 
-  //   if (newExp != m2) { 
-  //     success   = 1;
-  //     newExpVar = ctr->muExp(newExp);    
-  //     newTree   = new Node(*dlmTree2);   
-  //     newTree->setUpdate(1);
-  //     newDlmTerm2   = newTree->listTerminal();
-
-  //     for (Node* nt : newDlmTerm2)
-  //       Exp[newExp]->updateNodeVals(nt);
-
-  //     if ((ctr->interaction) && ((ctr->interaction == 2) || (newExp != m1))) {
-  //       if (newExp <= m1)
-  //         newMixVar = ctr->muMix(m1, newExp);
-  //       else
-  //         newMixVar = ctr->muMix(newExp, m1);
-  //     } else {
-  //       newMixVar = 0;
-  //     }
-  //   }
-  // }
-
-  // // * dlmTree 2 MHR 
-  // // StepMHR: Transition ratio
-  // // mhr0 & mhr: Likelihood of R
-  // // If successful, calculate MH ratio
-  // if (success) {
-  //   newDlmTerm2 = dlmTree2->listTerminal(1);
-  //   modTree->setUpdateXmat(1);
-
-  //   // MH ratio with a new terminal and the exposure: newDlmTerm1, newExpVar
-  //   mhr = dlmtreeHDLMM_MHR(modTerm, 
-  //                          dlmTerm1, newDlmTerm2, ctr, ZtR, treeVar, 
-  //                          m1Var, newExpVar, newMixVar);
-
-  //   // MH ratio - dlmTree
-  //   if (RtR < 0) {
-  //     RtR = (ctr->R).dot(ctr->R);
-  //     RtZVgZtR = ZtR.dot((ctr->Vg).selfadjointView<Eigen::Lower>() * ZtR);
-  //   }
-
-  //   ratio = stepMhr + 
-  //           mhr.logVThetaChol - mhr0.logVThetaChol -
-  //           (0.5 * (ctr->n + 1.0) *
-  //           (log(0.5 * (RtR - RtZVgZtR - mhr.beta) + ctr->xiInvSigma2) -
-  //           log(0.5 * (RtR - RtZVgZtR - mhr0.beta) + ctr->xiInvSigma2))) -
-  //           (0.5 * ((mhr.nTerm2 * mhr0.nModTerm * log(treeVar * newExpVar)) -
-  //           (mhr0.nTerm2 * mhr0.nModTerm * log(treeVar * m2Var))));
-
-  //   // Interaction
-  //   if (newMixVar != 0){ 
-  //     ratio -= 0.5 * log(treeVar * newMixVar) * mhr0.nTerm1 * mhr.nTerm2 * mhr0.nModTerm;
-  //   }
-
-  //   if (mixVar != 0){ 
-  //     ratio += 0.5 * log(treeVar * mixVar) * mhr0.nTerm1 * mhr0.nTerm2 * mhr0.nModTerm;
-  //   }
-
-  //   if ((log(R::runif(0, 1)) < ratio) && (ratio == ratio)) {
-  //     mhr0 = mhr;
-  //     success = 2;
-
-  //     if (step2 == 3) { 
-  //       m2      = newExp;
-  //       m2Var   = newExpVar;
-  //       mixVar  = newMixVar;
-  //       dlmTree2->replaceNodeVals(newTree);
-  //     } else {
-  //       dlmTree2->accept();
-  //     }
-
-  //     dlmTerm2 = dlmTree2->listTerminal(); 
-  //     // No need to update Xmat for dlmtree accept
-  //     for (Node* n : modTerm) {
-  //       n->nodevals->updateXmat = 0;
-  //     }
-
-  //   } else { 
-  //     modTree->setUpdateXmat(1); 
-  //   }
-  // } 
-  // dlmTree2->reject();  
-
-  // if (newTree != 0)
-  //   delete newTree;
-  // newTree = 0;
-
-  // // * Record tree 2
-  // if (ctr->diagnostics) {
-  //   Eigen::VectorXd acc(9);
-  //   acc << ctr->record, t, 2, step2, success, m2, dlmTerm2.size(), stepMhr, ratio;
-  //   (dgn->treeDLMAccept).push_back(acc);
-  // }
-
 
   // [New tree proposal method] ------------------------------------------------------------------------------------------------
   // *** Propose a new TDLMM tree 1 ***
@@ -892,12 +673,12 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
           (0.5 * (ctr->n + 1.0) *
           (log(0.5 * (RtR - RtZVgZtR - mhr.beta) + ctr->xiInvSigma2) -
           log(0.5 * (RtR - RtZVgZtR - mhr0.beta) + ctr->xiInvSigma2))) -
-          (0.5 * (((mhr.nTerm1 + mhr.nTerm2) * mhr0.nModTerm * log(treeVar * newExpVar)) -
-          ((mhr0.nTerm1 + mhr0.nTerm2) * mhr0.nModTerm * log(treeVar * m1Var)))); 
+          (0.5 * ((mhr.nTerm1 * mhr0.nModTerm * log(treeVar * newExpVar)) -
+          (mhr0.nTerm1 * mhr0.nModTerm * log(treeVar * m1Var)))); 
 
   // Interaction
   if (newMixVar != 0){ 
-    ratio -= 0.5 * log(treeVar * newMixVar) * mhr.nTerm1 * mhr.nTerm2 * mhr.nModTerm;
+    ratio -= 0.5 * log(treeVar * newMixVar) * mhr.nTerm1 * mhr0.nTerm2 * mhr0.nModTerm;
   }
 
   if (mixVar != 0){ 
@@ -948,18 +729,8 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
   newMixVar = mixVar;
   stepMhr   = 0;
   success   = 1;
-
-  // *** Create a new tree for proposal ***
-  newTree = new Node(0, 1);               // Start from the root
-  newTree->nodestruct = expNS->clone();   // Construct nodestruct
-  drawTree(newTree, newTree, ctr->treePrior[0], ctr->treePrior[1]); // Grow a tree structure from the root
-  newTree->setUpdate(1);                  // Update
-  newDlmTerm2 = newTree->listTerminal();  // List the number of terminal nodes for the new tree
-  for (Node* nt : newDlmTerm2) {          // Go through the new terminal node
-    Exp[newExp]->updateNodeVals(nt);      // Update the calculations
-  }
-
-  // *** Propose a new dlmtree 1 ***
+  
+    // *** Propose a new dlmtree 2 ***
   step2     = 0;                          // Always propose
   newExp    = sampleInt(ctr->expProb);    // Sample an exposure for the new tree
   newExpVar = ctr->muExp(newExp);         // Find the exposure-specific variance for the new exposure
@@ -976,6 +747,17 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
   } else {
     newMixVar = 0;
   }
+  // *** Create a new tree for proposal ***
+  newTree = new Node(0, 1);               // Start from the root
+  newTree->nodestruct = expNS->clone();   // Construct nodestruct
+  drawTree(newTree, newTree, ctr->treePrior[0], ctr->treePrior[1]); // Grow a tree structure from the root
+  newTree->setUpdate(1);                  // Update
+  newDlmTerm2 = newTree->listTerminal();  // List the number of terminal nodes for the new tree
+  for (Node* nt : newDlmTerm2) {          // Go through the new terminal node
+    Exp[newExp]->updateNodeVals(nt);      // Update the calculations
+  }
+
+
   
   // MH ratio
   modTree->setUpdateXmat(1);
@@ -991,17 +773,18 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
     RtZVgZtR  = ZtR.dot((ctr->Vg).selfadjointView<Eigen::Lower>() * ZtR);
   }
 
+
   ratio = stepMhr + 
           mhr.logVThetaChol - mhr0.logVThetaChol -
           (0.5 * (ctr->n + 1.0) *
           (log(0.5 * (RtR - RtZVgZtR - mhr.beta) + ctr->xiInvSigma2) -
           log(0.5 * (RtR - RtZVgZtR - mhr0.beta) + ctr->xiInvSigma2))) -
-          (0.5 * (((mhr.nTerm1 + mhr.nTerm2) * mhr0.nModTerm * log(treeVar * newExpVar)) -
-          ((mhr0.nTerm1 + mhr0.nTerm2) * mhr0.nModTerm * log(treeVar * m2Var))));
+          (0.5 * ((mhr.nTerm2 * mhr0.nModTerm * log(treeVar * newExpVar)) -
+          (mhr0.nTerm2 * mhr0.nModTerm * log(treeVar * m2Var))));
 
   // Interaction
   if (newMixVar != 0){ 
-    ratio -= 0.5 * log(treeVar * newMixVar) * mhr.nTerm1 * mhr.nTerm2 * mhr.nModTerm;
+    ratio -= 0.5 * log(treeVar * newMixVar) * mhr0.nTerm1 * mhr.nTerm2 * mhr0.nModTerm;
   }
 
   if (mixVar != 0){ 
@@ -1061,6 +844,7 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
                             ctr, ZtR, treeVar,
                             m1Var, m2Var, mixVar);
                             
+
     ratio = stepMhr + mhr.logVThetaChol - mhr0.logVThetaChol -
       (0.5 * (ctr->n + 1.0) *
         (log(0.5 * (RtR - RtZVgZtR - mhr.beta) + ctr->xiInvSigma2) -
@@ -1121,7 +905,7 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
   (ctr->dlmTree1Exp)(t)   = m1;
   (ctr->dlmTree2Exp)(t)   = m2;
   (ctr->expCount)(m1)++;                 
-  (ctr->expCount)(m2)++;                 
+  (ctr->expCount)(m2)++;                
   (ctr->expInf)(m1)       += ((ctr->tau)(t)); 
   (ctr->expInf)(m2)       += ((ctr->tau)(t)); 
   (ctr->totTermExp)(m1)   += mhr0.nModTerm * mhr0.nTerm1;   
@@ -1239,11 +1023,11 @@ void dlmtreeHDLMMGaussian_TreeMCMC(int t, NodeStruct* expNS, Node* modTree,
 
 
 treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,  
-                         std::vector<Node*> dlmTerm1,
-                         std::vector<Node*> dlmTerm2,
-                         dlmtreeCtr* ctr, 
-                         Eigen::VectorXd ZtR, 
-                         double treeVar, double m1Var, double m2Var, double mixVar)
+                          std::vector<Node*> dlmTerm1,
+                          std::vector<Node*> dlmTerm2,
+                          dlmtreeCtr* ctr, 
+                          Eigen::VectorXd ZtR, 
+                          double treeVar, double m1Var, double m2Var, double mixVar)
 
 {
   std::size_t s; 
@@ -1257,9 +1041,9 @@ treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,
   int pXMod = modTerm.size();
 
   // Number of terminal nodes
-  int pXDlm1  = dlmTerm1.size();
-  int pXDlm2  = dlmTerm2.size();
-  int pXDlm   = pXDlm1 + pXDlm2;
+  int pXDlm1 = dlmTerm1.size();
+  int pXDlm2 = dlmTerm2.size();
+  int pXDlm = pXDlm1 + pXDlm2;
   int interaction = 0;
   if (mixVar != 0) { 
     pXDlm += pXDlm1 * pXDlm2; 
@@ -1272,27 +1056,27 @@ treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,
   // Rcout << "Define parameters for calculation \n";
   Eigen::MatrixXd Xd, ZtX, VgZtX;
   Eigen::VectorXd diagVar;
-  Xd.resize(ctr->n, pXDlm);           Xd.setZero(); 
-  ZtX.resize(ctr->pZ, pXComb);        ZtX.setZero();
-  VgZtX.resize(ctr->pZ, pXComb);      VgZtX.setZero();
-  diagVar.resize(pXDlm);              diagVar.setZero(); 
+  Xd.resize(ctr->n, pXDlm);         Xd.setZero(); 
+  ZtX.resize(ctr->pZ, pXComb);      ZtX.setZero();
+  VgZtX.resize(ctr->pZ, pXComb);    VgZtX.setZero();
+  diagVar.resize(pXDlm);            diagVar.setZero(); 
   
   // Building exposure matrix, Xd & ZtX
   int i, j, k;
 
   // dlmTree 1
   for (i = 0; i < pXDlm1; i++) {               
-    Xd.col(i)   = (dlmTerm1[i]->nodevals)->X;
-    diagVar(i)  = 1.0 / (m1Var * treeVar);
-    ZtX.col(i)  = (dlmTerm1[i]->nodevals)->ZtX;
+    Xd.col(i) = (dlmTerm1[i]->nodevals)->X;
+    diagVar(i) = 1.0 / (m1Var * treeVar);
+    ZtX.col(i) = (dlmTerm1[i]->nodevals)->ZtX;
   }
 
   // dlmTree 2
   for (j = 0; j < pXDlm2; j++) {               
     k = pXDlm1 + j;                            
-    Xd.col(k)   = (dlmTerm2[j]->nodevals)->X; 
-    diagVar(k)  = 1.0 / (m2Var * treeVar);      
-    ZtX.col(k)  = (dlmTerm2[j]->nodevals)->ZtX;
+    Xd.col(k) = (dlmTerm2[j]->nodevals)->X; 
+    diagVar(k) = 1.0 / (m2Var * treeVar);      
+    ZtX.col(k) = (dlmTerm2[j]->nodevals)->ZtX;
   }
 
   // Mixture & interaction
@@ -1300,9 +1084,9 @@ treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,
     for (i = 0; i < pXDlm1; i++) {
       for (j = 0; j < pXDlm2; j++) {
         k = (pXDlm1 + pXDlm2) + i * pXDlm2 + j; 
-        Xd.col(k)   = (((dlmTerm1[i]->nodevals)->X).array() * ((dlmTerm2[j]->nodevals)->X).array()).matrix();
-        diagVar(k)  = 1.0 / (mixVar * treeVar);
-        ZtX.col(k)  = ctr->Zw.transpose() * Xd.col(k); 
+        Xd.col(k) = (((dlmTerm1[i]->nodevals)->X).array() * ((dlmTerm2[j]->nodevals)->X).array()).matrix();
+        diagVar(k) = 1.0 / (mixVar * treeVar);
+        ZtX.col(k) = ctr->Zw.transpose() * Xd.col(k); 
       }
     }
   }
@@ -1343,31 +1127,31 @@ treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,
 
     // Exposure 1 return
     // Rcout << "pxMod == 1: Exposure 1 segment \n";
-    draw1Temp   = ThetaDraw.head(pXDlm1);            
+    draw1Temp = ThetaDraw.head(pXDlm1);            
     out.term1T2 = (draw1Temp).dot(draw1Temp);      
-    out.nTerm1  = double(pXDlm1);                   
+    out.nTerm1 = double(pXDlm1);                   
 
     // Exposure 2 return
     // Rcout << "pxMod == 1: Exposure 2 segment \n";
-    draw2Temp   = ThetaDraw.segment(pXDlm1, pXDlm2); 
+    draw2Temp = ThetaDraw.segment(pXDlm1, pXDlm2); 
     out.term2T2 = (draw2Temp).dot(draw2Temp);      
-    out.nTerm2  = double(pXDlm2);                   
+    out.nTerm2 = double(pXDlm2);                   
 
     // Mixture return
     if (interaction) {    
       // Rcout << "pxMod == 1: Interaction/Mixture segment \n";
-      out.drawMix   = ThetaDraw.tail(pXDlm - pXDlm1 - pXDlm2);  // Extract last (pX1 x pX2) element for mixture
-      out.mixT2     = (out.drawMix).dot(out.drawMix);             // Mixture dot product for MHR math
-      out.nTermMix  = double(pXDlm - pXDlm1 - pXDlm2);
+      out.drawMix = ThetaDraw.tail(pXDlm - pXDlm1 - pXDlm2);  // Extract last (pX1 x pX2) element for mixture
+      out.mixT2 = (out.drawMix).dot(out.drawMix);             // Mixture dot product for MHR math
+      out.nTermMix = double(pXDlm - pXDlm1 - pXDlm2);
     }
 
     // Calculate and store parameters for MHR math 
     // Rcout << "pxMod == 1: MH ratio math calculation \n";
     out.fitted = Xd * out.drawAll;  // exposure matrix x delta
 
-    out.beta          = ThetaHat.dot(XtVzInvR);  // The big chunk in Eq(8)
+    out.beta = ThetaHat.dot(XtVzInvR);  // The big chunk in Eq(8)
     out.logVThetaChol = VThetaChol.diagonal().array().log().sum(); // |V_theta_a|
-    out.termT2        = (out.drawAll).dot(out.drawAll);
+    out.termT2 = (out.drawAll).dot(out.drawAll);
     
     out.nDlmTerm = pXDlm * 1.0; 
     out.nModTerm = pXMod * 1.0;    // = 1
@@ -1394,26 +1178,26 @@ treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,
     // Loop through indices and (subset) update Xtemp, Ztemp, Rtemp
     j = 0;
     for (int i : n->nodevals->idx) {
-      Xtemp.row(j)  = Xd.row(i);  
-      Ztemp.row(j)  = ctr->Z.row(i); 
-      Rtemp(j)      = ctr->R(i);         
+      Xtemp.row(j) = Xd.row(i);  
+      Ztemp.row(j) = ctr->Z.row(i); 
+      Rtemp(j) = ctr->R(i);         
       j++;
     } // end loop over node indices
       
     n->nodevals->XtX.resize(pXDlm, pXDlm);
-    n->nodevals->XtX        = Xtemp.transpose() * Xtemp;
+    n->nodevals->XtX = Xtemp.transpose() * Xtemp;
     n->nodevals->ZtXmat.resize(ctr->pZ, pXDlm);
-    n->nodevals->ZtXmat     = Ztemp.transpose() * Xtemp;
+    n->nodevals->ZtXmat = Ztemp.transpose() * Xtemp;
     n->nodevals->VgZtXmat.resize(ctr->pZ, pXDlm);
-    n->nodevals->VgZtXmat   = ctr->Vg * n->nodevals->ZtXmat;
+    n->nodevals->VgZtXmat = ctr->Vg * n->nodevals->ZtXmat;
     n->nodevals->updateXmat = 0;
     
     XtR.segment(start, pXDlm) = Xtemp.transpose() * Rtemp;
       
     // Update blocks
-    XtXblock.block(start, start, pXDlm, pXDlm)  = ((n->nodevals->XtX) + LInv).inverse();
-    ZtX.block(0, start, ctr->pZ, pXDlm)         = n->nodevals->ZtXmat;
-    VgZtX.block(0, start, ctr->pZ, pXDlm)       = n->nodevals->VgZtXmat;
+    XtXblock.block(start, start, pXDlm, pXDlm) = ((n->nodevals->XtX) + LInv).inverse();
+    ZtX.block(0, start, ctr->pZ, pXDlm) = n->nodevals->ZtXmat;
+    VgZtX.block(0, start, ctr->pZ, pXDlm) = n->nodevals->VgZtXmat;
     
     // Move to the next block
     start += pXDlm;
@@ -1421,17 +1205,19 @@ treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,
 
   // Compute elements of MH ratio
   // Rcout << "pxMod != 1: VTheta / ThetaHat calculation \n";
-  Eigen::MatrixXd ZtXXi       = ZtX * XtXblock;
-  Eigen::MatrixXd VTheta      = XtXblock;
-  VTheta.noalias() += ZtXXi.transpose() * (ctr->VgInv - ZtXXi * ZtX.transpose()).inverse() * ZtXXi;
-  Eigen::VectorXd XtVzInvR    = XtR;
+  Eigen::MatrixXd ZtXXi = ZtX * XtXblock;
+  Eigen::MatrixXd VTheta = XtXblock;
+  VTheta.noalias() += ZtXXi.transpose() * 
+    (ctr->VgInv - ZtXXi * ZtX.transpose()).inverse() * ZtXXi;
+  Eigen::VectorXd XtVzInvR = XtR;
   XtVzInvR.noalias() -= VgZtX.transpose() * ZtR;
-  Eigen::VectorXd ThetaHat    = VTheta * XtVzInvR;
-  Eigen::MatrixXd VThetaChol  = VTheta.llt().matrixL();
+  Eigen::VectorXd ThetaHat = VTheta * XtVzInvR;
+  Eigen::MatrixXd VThetaChol = VTheta.llt().matrixL();
 
   // Store the sampled values and also add variance 
   out.drawAll = ThetaHat;
-  out.drawAll.noalias() += VThetaChol * as<Eigen::VectorXd>(rnorm(pXComb, 0.0, sqrt(ctr->sigma2)));
+  out.drawAll.noalias() += 
+      VThetaChol * as<Eigen::VectorXd>(rnorm(pXComb, 0.0, sqrt(ctr->sigma2)));
 
   // drawAll = mod1-dlm1 / mod1-dlm2 / mod1-dlm1&2 / mod2-dlm1 / mod2-dlm2 / mod2-dlm1&2 / mod3 ...
   // Fitted value & terminal effect draws
@@ -1448,25 +1234,25 @@ treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,
   // Store Term1T2/Term2T2/MixT2 & nTerm1/nTerm2
   out.term1T2 = 0;
   out.term2T2 = 0;
-  out.mixT2   = 0;
+  out.mixT2 = 0;
   
   for (s = 0; s < modTerm.size(); s++) { 
     // Extract draws from the block matrix for each modifier terminal node
     drawTemp = out.drawAll.segment(s * pXDlm, pXDlm); 
 
     // Tree 1 draw 
-    draw1Temp   = drawTemp.head(pXDlm1);
+    draw1Temp = drawTemp.head(pXDlm1);
     out.term1T2 += (draw1Temp).dot(draw1Temp);
 
     // Tree 2 draw
-    draw2Temp   = drawTemp.segment(pXDlm1, pXDlm2);
+    draw2Temp = drawTemp.segment(pXDlm1, pXDlm2);
     out.term2T2 += (draw2Temp).dot(draw2Temp);
 
     // [Mixture] Tree 1 x Tree 2 draw
     if (interaction) {
-      drawMixTemp   = drawTemp.tail(pXDlm - pXDlm1 - pXDlm2);
-      out.mixT2     += (drawMixTemp).dot(drawMixTemp);
-      out.nTermMix  = double(pXDlm - pXDlm1 - pXDlm2);
+      drawMixTemp = drawTemp.tail(pXDlm - pXDlm1 - pXDlm2);
+      out.mixT2 += (drawMixTemp).dot(drawMixTemp);
+      out.nTermMix = drawMixTemp.size();
     }
 
     // Calculate fitted for each modifier terminal
@@ -1477,14 +1263,14 @@ treeMHR dlmtreeHDLMM_MHR(std::vector<Node*> modTerm,
 
   // Calculation return
   // Rcout << "pxMod != 1: MH ratio math calculation \n";
-  out.beta          = ThetaHat.dot(XtVzInvR);
+  out.beta = ThetaHat.dot(XtVzInvR);
   out.logVThetaChol = VThetaChol.diagonal().array().log().sum();
-  out.termT2        = (out.drawAll).dot(out.drawAll);
+  out.termT2 = (out.drawAll).dot(out.drawAll);
 
-  out.nTerm1    = double(pXDlm1);
-  out.nTerm2    = double(pXDlm2);
-  out.nDlmTerm  = pXDlm * 1.0; 
-  out.nModTerm  = pXMod * 1.0;
+  out.nTerm1 = double(pXDlm1);
+  out.nTerm2 = double(pXDlm2);
+  out.nDlmTerm = pXDlm * 1.0; 
+  out.nModTerm = pXMod * 1.0;
 
   return(out);
 }
